
require "class"
require "prefabnames"

local ConsoleSelect = Class(function(self)
	self.ents = {}
end)

function c_all(prefab)
	local query = ConsoleSelect()
	query.ents = Ents
	if prefab then
		return query:prefab(prefab)
	end
	return query
end

function c_players(name)
	local query = ConsoleSelect()
	for i,v in ipairs(AllPlayers) do
		if not name or v.name == name then
		    query.ents[v.GUID] = Ents[v.GUID]
		end
	end
	return query
end

function c_me()
	local query = ConsoleSelect()
	local inst = ThePlayer or AllPlayers[1]
	query.ents[inst.name] = inst
	return query
end

function c_mouse()
	local query = ConsoleSelect()
	local inst = ConsoleWorldEntityUnderMouse()
	query.ents[inst.name] = inst
	return query
end

function ConsoleSelect:print()
	for k,v in pairs(self.ents) do
		print(tostring(v))
	end
	return self
end

function ConsoleSelect:count()
	local count = 0
	for k,v in pairs(self.ents) do
		count = count + 1
	end
	print(count, "objects")
end

function ConsoleSelect:move(location)
	local x,y,z = c_getlocation(location)
	for k,v in pairs(self.ents) do
		if v.Transform then
			v.Transform:SetPosition(x,y,z)
		end
	end
	return self
end

function ConsoleSelect:give(prefab, count)
	prefab = c_prefab(prefab)
	for k,v in pairs(self.ents) do
		if v.components.inventory then
			for i=1, count or 1 do
				local inst = DebugSpawn(prefab)
				v.components.inventory:GiveItem(inst)
			end
		end
	end
	return self
end

function c_getlocation(location)
	if not location then
		return ConsoleWorldPosition():Get()
	end
	if location.getlocation then
		return location:getlocation()
	end
	return location.Get()	
end

function ConsoleSelect:getlocation()
	for k,v in pairs(self.ents) do
		return v.Transform:GetWorldPosition()
	end
end

function ConsoleSelect:delete()
	for k,v in pairs(self.ents) do
		v:Remove()
	end
end

function ConsoleSelect:addtag(tag)
	for k,v in pairs(self.ents) do
		v:AddTag(tag)
	end
end

function ConsoleSelect:prefab(prefab)
	return self:applyfilter(function(v)
		if not v.prefab then return false end
		return v.prefab == prefab or c_name(v.prefab) == prefab	
	end)
end

function ConsoleSelect:tag(tag)
	return self:applyfilter(function(v)
		return v:HasTag(tag)
	end)
end

function ConsoleSelect:applyfilter(filter)
	local query = ConsoleSelect()
	for k,v in pairs(self.ents) do
		if filter(v) then
			query.ents[k] = v
		end
	end
	return query
end

function ConsoleSelect:nearest()
	local query = ConsoleSelect()
	local player = ConsoleCommandPlayer()
	local closest = nil
	local closeness = nil
	for k,v in pairs(self.ents) do
		if closest == nil or player:GetDistanceSqToInst(v) < closeness then
			closest = v
			closeness = player:GetDistanceSqToInst(v)
		end
	end
	query.ents[closest.name] = closest
	return query
end
